﻿using System;
namespace TemplateMethod.Ejm1
{
    public abstract class CreditCard
    {
        protected string CardNum;
        protected int ExpMM, ExpYY;

        public CreditCard(string num, int expMonth, int expYear)
        {
            CardNum = num;
            ExpMM = expMonth;
            ExpYY = expYear;
        }

        public bool IsExpDtValid()
        {
            DateTime cal = DateTime.Now;
            //cal.setTime(new Date());
            int mm = cal.Month;
            int yy = cal.Year;
            bool result = (yy > ExpYY) || ((yy == ExpYY) && (mm > ExpMM));
            return (!result);
        }

        private bool HasValidChars()
        {
            string validChars = "0123456789";
            bool result = true;
            for (int i = 0; i < CardNum.Length; i++)
            {
                if (validChars.IndexOf(CardNum.Substring(i, 1)) < 0)
                {
                    result = false;
                    break;
                }
            }
            return result;
        }

        private bool IsValidCheckSum()
        {
            bool result = true;
            int sum = 0;
            int multiplier = 1;
            int strLen = CardNum.Length;
            for (int i = 0; i < strLen; i++)
            {
                //string digit = cardNum.Substring(strLen - i - 1, strLen - i);

                string digit = CardNum.Substring(strLen - i - 1, 1);

                int currProduct = Convert.ToInt16(digit) * multiplier;
                if (currProduct >= 10)
                    sum += (currProduct % 10) + 1;
                else
                    sum += currProduct;
                if (multiplier == 1)
                    multiplier++;
                else
                    multiplier--;
            }
            if ((sum % 10) != 0)
                result = false;
            return result;
        }

        /* methods to be overridden by sub-classes. */
        public abstract bool IsNumOfDigitsValid();

        public abstract bool IsValidPrefix();

        public abstract bool IsAccountInGoodStand();

        /* sealed method - subclasses cannot override
            ***TEMPLATE METHOD***
          public sealed override  bool isValid()?? No funciono sealed override
        */
        public bool IsValid()
        {
            if (!IsExpDtValid())
            {
                Console.WriteLine(" Invalid Expiration Date. ");
                return false;
            }
            if (!IsNumOfDigitsValid())
            {
                Console.WriteLine(" Invalid Number of Digits ");
                return false;
            }
            if (!IsValidPrefix())
            {
                Console.WriteLine(" Invalid Prefix ");
                return false;
            }
            if (!HasValidChars())
            {
                Console.WriteLine(" Invalid Characters ");
                return false;
            }
            if (!IsValidCheckSum())
            {
                Console.WriteLine(" Invalid Check Sum ");
                return false;
            }
            if (!IsAccountInGoodStand())
            {
                Console.WriteLine(" Account is Inactive/Revoked/Over the Limit ");
                return false;
            }
            return true;
        }
    }
}
